@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -9,23 +9,18 @@
 #ifndef SQUID_STOREMETASTDLFS_H
 #define SQUID_STOREMETASTDLFS_H
 
-/* for inheritance */
 #include "StoreMeta.h"
-/* for MEMPROXY() macros */
-#include "MemPool.h"
 
 class StoreMetaSTDLFS : public StoreMeta
 {
-public:
     MEMPROXY_CLASS(StoreMetaSTDLFS);
 
+public:
     char getType() const {return STORE_META_STD_LFS;}
 
     bool validLength(int) const;
     //    bool checkConsistency(StoreEntry *) const;
 };
 
-MEMPROXY_CLASS_INLINE(StoreMetaSTDLFS);
-
 #endif /* SQUID_STOREMETASTDLFS_H */
 
