@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -9,22 +9,17 @@
 #ifndef SQUID_STOREMETAURL_H
 #define SQUID_STOREMETAURL_H
 
-/* for inheritance */
 #include "StoreMeta.h"
-/* for MEMPROXY_CLASS() macros */
-#include "MemPool.h"
 
 class StoreMetaURL : public StoreMeta
 {
-public:
     MEMPROXY_CLASS(StoreMetaURL);
 
+public:
     char getType() const {return STORE_META_URL;}
 
     bool checkConsistency(StoreEntry *) const;
 };
 
-MEMPROXY_CLASS_INLINE(StoreMetaURL);
-
 #endif /* SQUID_STOREMETAURL_H */
 
