@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -9,6 +9,7 @@
 #include "squid.h"
 #include "auth/basic/Config.h"
 #include "auth/basic/User.h"
+#include "auth/CredentialsCache.h"
 #include "Debug.h"
 #include "SquidConfig.h"
 #include "SquidTime.h"
@@ -79,3 +80,16 @@ Auth::Basic::User::updateCached(Auth::Basic::User *from)
     }
 }
 
+CbcPointer<Auth::CredentialsCache>
+Auth::Basic::User::Cache()
+{
+    static CbcPointer<Auth::CredentialsCache> p(new Auth::CredentialsCache("basic", "GC Basic user credentials"));
+    return p;
+}
+
+void
+Auth::Basic::User::addToNameCache()
+{
+    Cache()->insert(userKey(), this);
+}
+
