@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -13,14 +13,14 @@
 
 #include "comm/forward.h"
 #include "defines.h"
-#include "hier_code.h"
-#include "ip/Address.h"
-#include "MemPool.h"
-#include "typedefs.h"
 #if USE_SQUID_EUI
 #include "eui/Eui48.h"
 #include "eui/Eui64.h"
 #endif
+#include "hier_code.h"
+#include "ip/Address.h"
+#include "ip/forward.h"
+#include "mem/forward.h"
 #include "SquidTime.h"
 
 #include <iosfwd>
@@ -28,6 +28,11 @@
 
 class CachePeer;
 
+namespace Security
+{
+class NegotiationHistory;
+};
+
 namespace Comm
 {
 
@@ -59,9 +64,9 @@ namespace Comm
  */
 class Connection : public RefCountable
 {
-public:
     MEMPROXY_CLASS(Comm::Connection);
 
+public:
     Connection();
 
     /** Clear the connection properties and close any open socket. */
@@ -100,7 +105,17 @@ public:
     /** The time the connection started */
     time_t startTime() const {return startTime_;}
 
+    /** The connection lifetime */
+    time_t lifeTime() const {return squid_curtime - startTime_;}
+
+    /** The time left for this connection*/
+    time_t timeLeft(const time_t idleTimeout) const;
+
     void noteStart() {startTime_ = squid_curtime;}
+
+    Security::NegotiationHistory *tlsNegotiations();
+    const Security::NegotiationHistory *hasTlsNegotiations() const {return tlsHistory;}
+
 private:
     /** These objects may not be exactly duplicated. Use copyDetails() instead. */
     Connection(const Connection &c);
@@ -143,12 +158,13 @@ private:
 
     /** The time the connection object was created */
     time_t startTime_;
+
+    /** TLS connection details*/
+    Security::NegotiationHistory *tlsHistory;
 };
 
 }; // namespace Comm
 
-MEMPROXY_CLASS_INLINE(Comm::Connection);
-
 // NP: Order and namespace here is very important.
 //     * The second define inlines the first.
 //     * Stream inheritance overloading is searched in the global scope first.
