@@ -1,8 +1,13 @@
 /*
- * DEBUG: section 16    Cache Manager API
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
  */
 
+/* DEBUG: section 16    Cache Manager API */
+
 #include "squid.h"
 #include "base/TextException.h"
 #include "ipc/Messages.h"
@@ -16,7 +21,7 @@ void DumpServiceTimesStats(Mgr::ServiceTimesActionData& stats, StoreEntry* sentr
 
 Mgr::ServiceTimesActionData::ServiceTimesActionData()
 {
-    xmemset(this, 0, sizeof(*this));
+    memset(this, 0, sizeof(*this));
 }
 
 Mgr::ServiceTimesActionData&
@@ -55,8 +60,8 @@ Mgr::ServiceTimesAction::Create(const CommandPointer &cmd)
     return new ServiceTimesAction(cmd);
 }
 
-Mgr::ServiceTimesAction::ServiceTimesAction(const CommandPointer &cmd):
-        Action(cmd), data()
+Mgr::ServiceTimesAction::ServiceTimesAction(const CommandPointer &aCmd):
+    Action(aCmd), data()
 {
     debugs(16, 5, HERE);
 }
@@ -96,3 +101,4 @@ Mgr::ServiceTimesAction::unpack(const Ipc::TypedMsgHdr& msg)
     msg.checkType(Ipc::mtCacheMgrResponse);
     msg.getPod(data);
 }
+
