@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -11,7 +11,7 @@
 
 #include "globals.h"
 #include "profiler/Profiler.h"
-#include "SBufList.h"
+#include "sbuf/List.h"
 
 /** A list of C-strings
  *
@@ -48,7 +48,7 @@ const char *wordlistAdd(wordlist **, const char *);
 
 /** Concatenate a wordlist
  *
- * \deprecated use SBufListContainerJoin(SBuf()) from SBufAlgos.h instead
+ * \deprecated use SBufListContainerJoin(SBuf()) from sbuf/Algorithms.h instead
  */
 void wordlistCat(const wordlist *, MemBuf *);
 
@@ -60,7 +60,7 @@ void wordlistAddWl(wordlist **, wordlist *);
 
 /** Concatenate the words in a wordlist
  *
- * \deprecated use SBufListContainerJoin(SBuf()) from SBufAlgos.h instead
+ * \deprecated use SBufListContainerJoin(SBuf()) from sbuf/Algorithms.h instead
  */
 void wordlistJoin(wordlist **, wordlist **);
 
